import aiohttp


class AsyncHTTPClient:
    def __init__(self, base_url: str, timeout: int = 60):
        self.base_url = base_url
        self.timeout = timeout
        self.headers = {
            "content-type": "application/json",
        }

    def set_auth_header(self, api_key: str):
        self.headers["Authorization"] = f"Bearer {api_key}"

    async def get(self, path: str, params: dict = None) -> dict:
        url = self.base_url + path
        async with aiohttp.ClientSession() as session:
            async with session.get(
                url, params=params, timeout=self.timeout, headers=self.headers
            ) as response:
                return await self._handle_response(response)

    async def post(self, path: str, data: dict = None, headers: dict = None, **kwargs) -> dict:
        url = self.base_url + path
        _headers = self.headers
        if headers:
            _headers.update(headers)
        async with aiohttp.ClientSession() as session:
            async with session.post(
                url, data=data, timeout=self.timeout, headers=_headers, **kwargs
            ) as response:
                return await self._handle_response(response)

    async def put(self, path: str, data: dict = None) -> dict:
        url = self.base_url + path
        async with aiohttp.ClientSession() as session:
            async with session.put(
                url, data=data, timeout=self.timeout, headers=self.headers
            ) as response:
                return await self._handle_response(response)

    async def delete(self, path: str, data: dict = None) -> dict:
        url = self.base_url + path
        async with aiohttp.ClientSession() as session:
            async with session.delete(
                url, data=data, timeout=self.timeout, headers=self.headers
            ) as response:
                return await self._handle_response(response)

    async def patch(self, path: str, data: dict = None) -> dict:
        url = self.base_url + path
        async with aiohttp.ClientSession() as session:
            async with session.patch(
                url, data=data, timeout=self.timeout, headers=self.headers
            ) as response:
                return await self._handle_response(response)

    async def _handle_response(self, response: aiohttp.ClientResponse) -> dict:
        response.raise_for_status()
        return await response.json()



import uuid
from enum import Enum
from typing import Optional

from pydantic import BaseModel, ConfigDict

from vendi.core.schema import SchemaMixin


class TrainStatus(str, Enum):
    SCHEDULED = "SCHEDULED"
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"
    CRASHED = "CRASHED"
    PAUSED = "PAUSED"
    CANCELLING = "CANCELLING"


class TrainParameters(BaseModel):
    model_max_length: Optional[int] = 1024
    split_eval: Optional[float] = 0.1
    eval_steps: Optional[int] = 10
    save_steps: Optional[int] = 50
    per_device_train_batch_size: Optional[int] = 1
    per_device_eval_batch_size: Optional[int] = 1
    num_train_epochs: Optional[int] = 3
    max_steps: Optional[int] = 50
    save_total_limit: Optional[int] = 5
    load_best_model_at_end: Optional[bool] = True
    learning_rate: Optional[float] = 0.00002
    gradient_accumulation_steps: Optional[int] = 1
    warmup_steps: Optional[int] = 0
    warmup_ratio: Optional[float] = 0.03

    model_config = ConfigDict(
        protected_namespaces=(),
    )


class LoraTrainParams(TrainParameters):
    lora_r: Optional[int] = 32
    lora_alpha: Optional[int] = 32
    lora_dropout: Optional[float] = 0.05
    lora_target_modules: list[str] | None = [
        "q_proj",
        "o_proj",
        "up_proj",
        "v_proj",
        "gate_proj",
        "down_proj",
        "k_proj",
        "lm_head"
    ]
    q_lora: bool = True


class TrainData(BaseModel):
    logs: list[dict]
    chart: list[dict]
    run_config: dict


class CreateTrainJobSchema(BaseModel):
    model_id: uuid.UUID
    prefect_flow_id: uuid.UUID
    type_: str
    status: TrainStatus
    dataset: str
    base_model: str
    compute_resource: str
    training_params: dict


class TrainJob(CreateTrainJobSchema, SchemaMixin):
    pass



import asyncio
import time
import uuid
from typing import List, Dict, Optional

from vendi.completions.schema import ChatCompletion, ModelParameters, BatchInference, BatchInferenceStatus, \
    CompletionRequest, Endpoint, VendiCompletionResponse
from vendi.core.ahttp_client import AsyncHTTPClient
from vendi.core.http_client import HttpClient


class Completions:
    """
    Completions is the client to interact with the completions endpoint of the Vendi API.
    """

    def __init__(self, url: str, api_key: str):
        """
        Initialize the Completions client
        :param url: The URL of the Vendi API
        :param api_key: The API key to use for authentication
        """
        self.__api_key = api_key
        self.__client = HttpClient(
            url=url,
            api_key=api_key,
        )
        self.__aclient = AsyncHTTPClient(
            base_url=url,
        )
        self.__aclient.set_auth_header(api_key)

    def create(
        self,
        model: str,
        messages: List[Dict[str, str]],
        frequency_penalty: Optional[float] = 0,
        presence_penalty: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        stop: Optional[List[str]] = None,
        n: Optional[int] = 1,
        top_p: Optional[float] = 1,
        top_k: Optional[int] = 40,
        temperature: Optional[float] = 0.7,
        json_schema: Optional[str] = None,
        regex: Optional[str] = None,
        checkpoint: str | None = None,
        request_id: str = None,
        openai_compatible: bool = False,
        extra_body: Optional[Dict] = None,
        extra_headers: Optional[Dict] = None,
    ) -> VendiCompletionResponse | ChatCompletion:
        """
        Create a completion on a language model with the given parameters
        :param model: The ID of the language model to use for the completion. Should be in the format of <provider>/<model_id>
        :param messages: The messages to use as the prompt for the completion
        :param frequency_penalty: The frequency penalty to use for the completion
        :param presence_penalty: The presence penalty to use for the completion
        :param max_tokens: The maximum number of tokens to generate for the completion
        :param stop: The stop condition to use for the completion
        :param n: The number of completions to generate
        :param top_p: The top p value to use for the completion
        :param top_k: The top k value to use for the completion
        :param temperature: The temperature value to use for the completion
        :param json_schema: The JSON schema to use for the completion (either this or regex should be provided) .
        See https://docs.vendi-ai.com/docs/completions-api#json-schema for more information
        :param regex: The regex to use for the completion (either this or json_schema should be provided)
        See https://docs.vendi-ai.com/docs/completions-api#regex for more information
        :param checkpoint: The checkpoint to use for the completion (either "latest" or a specific checkpoint ID for lora adapters)
        :param request_id: The request ID to use for the completion (optional)
        :param openai_compatible: Whether to return the response in OpenAI compatible format or the vendi full response format
        :param extra_body: Extra body parameters to include in the request body
        :param extra_headers: Extra headers to include in the request
        :return: The generated completion

        """
        data = {
            "messages": messages,
            "model": model,
            "frequency_penalty": frequency_penalty,
            "presence_penalty": presence_penalty,
            "max_tokens": max_tokens,
            "n": n,
            "top_p": top_p,
            "top_k": top_k,
            "temperature": temperature,
            "json_schema": json_schema,
            "regex": regex,
            "checkpoint": checkpoint,
            "request_id": request_id,
            "openai_compatible": openai_compatible,
        }
        if extra_body:
            data.update(extra_body)

        if stop is not None:
            data["stop"] = stop

        _res = self.__client.post(
            uri=f"/v1/chat/completions",
            json_data=data,
            headers=extra_headers
        )

        if openai_compatible:
            return ChatCompletion(**_res)
        return VendiCompletionResponse(**_res)

    async def acreate(
        self,
        model: str,
        messages: List[Dict[str, str]],
        frequency_penalty: Optional[float] = 0,
        presence_penalty: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        stop: Optional[List[str]] = None,
        n: Optional[int] = 1,
        top_p: Optional[float] = 1,
        top_k: Optional[int] = 40,
        temperature: Optional[float] = 0.7,
        json_schema: Optional[str] = None,
        regex: Optional[str] = None,
        checkpoint: str | None = None,
        request_id: str = None,
        openai_compatible: bool = False,
        extra_body: Optional[Dict] = None,
        extra_headers: Optional[Dict] = None,
    ) -> VendiCompletionResponse | ChatCompletion:
        """
        Same documentation as completions.create but with async operation
        """

        data = {
            "messages": messages,
            "model": model,
            "frequency_penalty": frequency_penalty,
            "presence_penalty": presence_penalty,
            "max_tokens": max_tokens,
            "n": n,
            "top_p": top_p,
            "top_k": top_k,
            "temperature": temperature,
            "json_schema": json_schema,
            "regex": regex,
            "checkpoint": checkpoint,
            "request_id": request_id,
            "openai_compatible": openai_compatible,
        }

        if extra_body:
            data.update(extra_body)

        if stop is not None:
            data["stop"] = stop

        _res = await self.__aclient.post(
            path=f"/v1/chat/completions",
            json=data,
            headers=extra_headers
        )
        if openai_compatible:
            return ChatCompletion(**_res)
        return VendiCompletionResponse(**_res)

    async def acreate_many(
        self,
        requests: list[CompletionRequest],
    ) -> List[ChatCompletion] | List[VendiCompletionResponse]:
        """
        Create multiple completions on different models with the same prompt and parameters
        requests: A list of completionr requests
        Examples:
        >>> import uuid
        >>> from vendi.completions.schema import CompletionRequest
        >>> from vendi import Vendi
        >>>
        >>> client = Vendi(api_key="my-api-key")
        >>> completions = asyncio.run(client.completions.acreate_many(
        >>>     requests=[
        >>>         CompletionRequest(
        >>>             model="openai/gpt-3.5-turbo",
        >>>             messages=[
        >>>                 {"role": "user", "content": "Hey how are you?"}
        >>>             ],
        >>>             request_id=str(uuid.uuid4()),
        >>>         ),
        >>>         CompletionRequest(
        >>>             model="openai/gpt-4",
        >>>             messages=[
        >>>                 {"role": "user", "content": "Hi whats up?"}
        >>>             ],
        >>>             request_id=str(uuid.uuid4()),
        >>>         ),
        >>>     ]
        >>>     )
        >>> )
        """

        tasks = [
            self.acreate(
                **request.model_dump(),
            ) for request in requests
        ]

        res = await asyncio.gather(*tasks)
        return res

    def available_endpoints(self) -> List[Endpoint]:
        """
        Get the list of available endpoints , those that are configured and ready to use
        :return: The list of available endpoints
        """
        _endpoints = self.__client.get(uri="/platform/v1/inference/endpoints")
        return [Endpoint(**endpoint) for endpoint in _endpoints]

    def endpoints_catalog(self) -> List[Endpoint]:
        """
        Get the list of all supported endpoints within the platform
        :return: The list of all supported endpoints
        """
        _endpoints = self.__client.get(uri="/platform/v1/inference/endpoints/all")
        return [Endpoint(**endpoint) for endpoint in _endpoints]

    def run_batch_job(
        self,
        dataset_id: uuid.UUID,
        model_parameters: list[ModelParameters],
        wait_until_complete: bool = False,
        timeout: int = 3000,
        poll_interval: int = 5,
    ) -> BatchInference:
        """
        Run a batch inference job on a dataset using the specified models .
        :param dataset_id: The ID of the dataset to run the batch inference on
        :param model_parameters: The list of model parameters to use for the batch inference
        :param wait_until_complete: Whether to wait until the batch inference job is complete before continuing
        :param timeout: The maximum time to wait in the client for the batch inference job to complete, in seconds. Valid only if wait_until_complete is True
        :param poll_interval: The interval at which to poll the batch inference job status, in seconds . Valid only if wait_until_complete is True
        :return: The batch inference job
        """

        job = self.__post_batch_job(dataset_id, model_parameters)
        if wait_until_complete:
            start_time = time.time()
            while True:
                status = self.batch_job_status(job.id)
                if status in [BatchInferenceStatus.COMPLETED, BatchInferenceStatus.FAILED]:
                    return job
                if time.time() - start_time > timeout:
                    raise TimeoutError(
                        "The batch job did not complete within the specified timeout. "
                        "You can still check its status by using the batch_job_status method.")

                time.sleep(poll_interval)
        return job

    async def arun_batch_job(
        self,
        dataset_id: uuid.UUID,
        model_parameters: list[ModelParameters],
        wait_until_complete: bool = False,
        timeout: int = 3000,
        poll_interval: int = 5,
    ) -> BatchInference:
        _res = await self.__aclient.post(
            path="/platform/v1/inference/batch/",
            json={
                "dataset_id": str(dataset_id),
                "model_parameters": [{**i.model_dump(), **i.model_extra} for i in model_parameters]
            }
        )
        job = BatchInference(**_res)

        if wait_until_complete:
            start_time = time.time()
            while True:
                job = await self._aget_batch_job(job.id)
                if job.status in [BatchInferenceStatus.COMPLETED, BatchInferenceStatus.FAILED]:
                    return job
                if time.time() - start_time > timeout:
                    raise TimeoutError(
                        "The batch job did not complete within the specified timeout. "
                        "You can still check its status by using the batch_job_status method.")

                await asyncio.sleep(poll_interval)

    def __post_batch_job(self, dataset_id: uuid.UUID, model_parameters: list[ModelParameters]) -> BatchInference:
        res = self.__client.post(
            uri="/platform/v1/inference/batch/",
            json_data={
                "dataset_id": str(dataset_id),
                "model_parameters": [{**i.model_dump(), **i.model_extra} for i in model_parameters]
            }
        )

        return BatchInference(**res)

    def batch_job_status(self, job_id: uuid.UUID) -> BatchInferenceStatus:
        """
        Returns the status of a batch inference job.
        """
        job = self._get_batch_job(job_id)
        return job.status

    def _get_batch_job(self, batch_inference_id: uuid.UUID) -> BatchInference:
        """
        Get a batch inference object job by ID
        """
        res = self.__client.get(
            uri=f"/platform/v1/inference/batch/{batch_inference_id}"
        )
        return BatchInference(**res)

    async def _aget_batch_job(self, batch_inference_id: uuid.UUID) -> BatchInference:
        """
        Get a batch inference object job by ID
        """
        res = await self.__aclient.get(
            path=f"/platform/v1/inference/batch/{batch_inference_id}"
        )
        return BatchInference(**res)

    def list_batch_jobs(self) -> List[BatchInference]:
        """
        Get all batch inferences
        """
        res = self.__client.get(
            uri="/platform/v1/inference/batch/"
        )
        return [BatchInference(**i) for i in res]

    def delete_batch_job(self, batch_id: uuid.UUID):
        """
        Delete a batch inference job
        """
        return self.__client.delete(f"/platform/v1/inference/batch/{batch_id}")



import uuid
from enum import Enum
from typing import Literal

from pydantic import BaseModel, ConfigDict

from vendi.core.schema import SchemaMixin


class DeploymentStatus(str, Enum):
    LIVE = "LIVE"
    """The deployment is live and ready to be used."""
    DOES_NOT_EXIST = "DOES_NOT_EXIST"
    """The deployment does not exist."""
    ERROR = "ERROR"
    """The deployment is in an error state."""
    UNKNOWN = "UNKNOWN"
    """The deployment is in an unknown state."""
    STOPPED = "STOPPED"
    """The deployment is stopped and not ready to be used."""
    STOPPING = "STOPPING"
    """The deployment is being stopped. This is a temporary state."""
    NOT_READY = "NOT_READY"
    """The deployment is not ready yet to be used."""


class DeploymentConfig(BaseModel):
    name: str
    """The name of the deployment."""
    namespace: str
    """The namespace of the deployment in the cluster."""
    version: str
    """The image version of the deployment."""
    scale_to_zero: bool
    """Whether or not to scale the deployment to zero when it is not in use. This value must be a boolean."""
    scale_to_zero_timeout_seconds: int
    """The number of seconds to wait before scaling the deployment to zero when it is not in use. This value must be between 0 and inf."""


class ModelConfig(BaseModel):
    model_name: str
    """The name of the model to deploy."""
    backend: Literal["pt", "ctranslate", "vllm", "gguf"] = "pt"
    """The backend to use for the model."""
    dtype: str = "float16"
    """The data type to use for the model. float16 is recommended for most models. For newer GPUs, bfloat16 is better."""
    gpu_memory_utilization: float = 0.8
    """The maximum percentage of GPU memory to use for the model."""
    max_model_len: int = 8192
    """The maximum length of the model to use for the deployment. This value must be between 0 and inf. Make sure this value is suitable for your model card on huggingface."""
    quantization: str | None = None
    """The quantization to use for the model. Please see the vendi documentation for a list of supported quantizations."""

    model_config = ConfigDict(
        protected_namespaces=(),
    )


class Deployment(SchemaMixin):
    name: str
    """The name of the deployment."""
    address: str
    """The address of the deployment in the cluster."""
    resource_name: str
    """The compute resource name of the deployment in the cluster."""
    provider: str
    """The provider of the deployment. vendi/azure/gcp/aws/tenant"""
    model_configuration: ModelConfig
    """The model configuration of the deployment."""
    deploy_configuration: DeploymentConfig
    """The deployment configuration of the deployment."""
    status: DeploymentStatus
    """The status of the deployment."""

    model_config = ConfigDict(
        protected_namespaces=(),
    )



import uuid
from enum import Enum
from typing import Callable, Any

import pandas as pd
from pydantic import BaseModel

from vendi.core.schema import SchemaMixin


class DatasetType(str, Enum):
    RAW = "raw"
    """A raw dataset is a dataset that has not been processed in any way. It is the most basic form of a dataset with random structure/dataset column names"""
    CONVERSATION = "conversation"
    """A conversation dataset is a dataset has a 'messages' key where each 'message' has 'role' and 'content' keys"""


class Dataset(SchemaMixin):
    name: str
    """The name of the dataset."""
    path: str
    """The path of the dataset."""
    storage: str | None = None
    """The storage the dataset is stored in."""
    _data: Any = None
    """The dataset data, only available after calling `load_data()`."""
    load_data: Callable[[], Any] | None = None
    """The function to call to load the dataset data. Populated after calling `get()` or `list()`."""

    @property
    def data(self) -> pd.DataFrame:
        """
        Get the dataset data as a pandas DataFrame
        """
        if self._data is None:
            raise ValueError("Dataset data is not loaded, run `load_data()` first")
        return self._data



from typing import List

from vendi.core.http_client import HttpClient

from .schema import Deployment, DeploymentStatus


class Deployments:
    def __init__(self, url: str, api_key: str):
        self.__api_key = api_key

        self.__client = HttpClient(
            url=url,
            api_key=api_key,
            api_prefix=f"/v1/"
        )

    def list(self) -> list[Deployment]:
        """
        Get a list of the deployments available to the user
        """
        res = self.__client.get(uri=f"deployments/")
        return [Deployment(**deployment) for deployment in res]

    def list_private(self) -> List[Deployment]:
        """
        Get a list of the private deployments available to the user
        """
        res = self.__client.get(uri=f"deployments/")
        return [Deployment(**deployment) for deployment in res]

    def get(self, deployment_id: str) -> Deployment:
        """
        Get deployment metadata by ID
        :param deployment_id: The ID of the deployment to get
        :return: The deployment metadata
        """
        res = self.__client.get(uri=f"deployments/{deployment_id}")
        return Deployment(**res)

    def delete(self, deployment_id: str) -> bool:
        """
        Delete a deployment by ID
        :param deployment_id: The ID of the deployment to delete
        :return: True if the deployment was deleted successfully
        """
        res = self.__client.delete(uri=f"deployments/{deployment_id}")
        return res

    def create(
        self,
        name: str,
        model_name: str,
        model_id: str,
        backend: str,
        dtype: str,
        gpu_memory_utilization: float,
        max_model_len: int,
        scale_to_zero: bool = True,
        scale_to_zero_timeout_seconds: int = 900,
        quantization: str = None,
    ) -> Deployment:
        """
        Create a deployment on vendi with the given parameters
        :param name: The name of the deployment
        :param model_name: The name of the model to deploy. This model must be registered with vendi models and available to the user when running
        vendi.models.list().
        :param model_id: The ID of the model to deploy. This model must be registered with vendi models and available to the user when running
        :param backend: The backend to use for the deployment. Please see the vendi documentation for a list of supported backends.
        :param dtype: The data type to use for the deployment. Please see the vendi documentation for a list of supported data types.
        :param gpu_memory_utilization: The percentage of GPU memory to use for the deployment. This value must be between 0 and 1.
        :param max_model_len: The maximum length of the model to use for the deployment. This value must be between 0 and inf.
        :param scale_to_zero: Whether or not to scale the deployment to zero when it is not in use. This value must be a boolean.
        A deployment not in use is defined by the deployment not receiving any requests for the scale_to_zero_timeout_seconds.
        :param scale_to_zero_timeout_seconds: The number of seconds to wait before scaling the deployment to zero when it is not in use.
        This value must be between 0 and inf.
        :param quantization: The quantization to use for the deployment. Please see the vendi documentation for a list of supported quantizations.
        :return: The created deployment metadata
        """
        res = self.__client.post(
            uri=f"deployments/create/",
            json_data={
                "name": name,
                "model_configuration": {
                    "model_name": model_name,
                    "backend": backend,
                    "dtype": dtype,
                    "gpu_memory_utilization": str(gpu_memory_utilization),
                    "max_model_len": str(max_model_len),
                    "quantization": quantization,
                },
                "model_id": model_id,
                "scale_to_zero": scale_to_zero,
                "scale_to_zero_timeout_seconds": scale_to_zero_timeout_seconds,
            },
        )
        return Deployment(**res)

    def update(
        self,
        deployment_id: str,
        scale_to_zero: bool | None = None,
        scale_to_zero_timeout_seconds: int | None = None,
    ) -> Deployment:
        """
        Update a deployment on vendi with the given parameters
        :param deployment_id: The ID of the deployment to update
        :param scale_to_zero: Whether or not to scale the deployment to zero when it is not in use. This value must be a boolean.
        A deployment not in use is defined by the deployment not receiving any requests for the scale_to_zero_timeout_seconds.
        :param scale_to_zero_timeout_seconds: The number of seconds to wait before scaling the deployment to zero when it is not in use.
        This value must be between 0 and inf.
        :return: The updated deployment metadata
        """
        res = self.__client.patch(
            uri=f"deployments/{deployment_id}",
            json_data={
                "scale_to_zero": scale_to_zero,
                "scale_to_zero_timeout_seconds": scale_to_zero_timeout_seconds,
            },
        )
        return Deployment(**res)

    def start(
        self,
        deployment_id: str,
    ) -> Deployment:
        """
        Start a deployment by ID from a stopped state
        :param deployment_id: The ID of the deployment to start
        :return: The updated deployment metadata
        """
        res = self.__client.post(
            uri=f"deployments/{deployment_id}/start",
        )
        return Deployment(**res)

    def stop(
        self,
        deployment_id: str,
    ) -> Deployment:
        """
        Stop a deployment by ID from a started state
        :param deployment_id: The ID of the deployment to stop
        :return: The updated deployment metadata
        """
        res = self.__client.post(
            uri=f"deployments/{deployment_id}/stop",
        )
        return Deployment(**res)

    def status(self, deployment_id: str) -> DeploymentStatus:
        """
        Get the status of a deployment
        :param deployment_id: The ID of the deployment to get the status of
        :return: The status of the deployment
        """
        res = self.__client.get(uri=f"deployments/{deployment_id}")
        return res["status"]



from vendi.core.http_client import HttpClient
from vendi.models.schema import Model, HuggingFaceModel, ModelProvider


class Models:
    def __init__(self, url: str, api_key: str):
        self.__client = HttpClient(
            url=url,
            api_key=api_key,
            api_prefix=f"/platform/v1/models"
        )

    # def create(self, name: str, model: Model):
    #     pass

    def import_model(
        self,
        model_name: str,
        model_provider: ModelProvider,
    ) -> Model:
        """
        Relevant only for Self-hosted customers.
        Import a model from any of the supported model providers.
        For huggingface, pass the model card name as the model_name.
        """
        res = self.__client.post(
            uri=f"/import",
            json_data={
                "model_name": model_name,
                "model_provider": model_provider,
            },
        )
        return Model(**res)

    def get(self, model_id: str):
        """
        Get mode metadata by ID
        """
        res = self.__client.get(uri=f"/{model_id}")
        return Model(**res)

    def list(self):
        """
        List all models available to the user
        """
        res = self.__client.get(uri=f"/")
        return [Model(**model) for model in res]

    def delete(self, model_id: str) -> bool:
        """
        Delete a model by ID
        """
        res = self.__client.delete(uri=f"/{model_id}")
        return res

    def set_public(self, model_id: str, status: bool) -> bool:
        """
        Set the public status of a model so that all vendi customers can use it.
        """
        res = self.__client.post(uri=f"/{model_id}/set-public/{status}")
        return res

    def get_huggingface_model(self, hf_model_name: str) -> HuggingFaceModel:
        """
        Get metadata information about a huggingface model card.
        :param hf_model_name: The model card name
        """
        res = self.__client.post(uri=f"/huggingface/get", json_data={"hf_model_name": hf_model_name})
        return HuggingFaceModel(**res)



from typing import Dict, Optional, Any

import requests
from requests import Response


class HttpClient:
    def __init__(self, api_key: str, url: str | None = None, api_prefix: Optional[str] = None):
        self.__url: str = url
        self.__api_url: str = url + api_prefix if api_prefix else url
        self.__api_key: str = api_key

    @property
    def base_url(self) -> str:
        return self.__url

    @property
    def api_url(self) -> str:
        return self.__api_url

    # @retry(wait=wait_exponential(max=3), stop=stop_after_attempt(3), reraise=True)
    def put(
        self,
        uri: str,
        json_data: Any = None,
        data: Any = None,
        headers: Optional[Dict] = None,
    ) -> Any:
        _headers = self._set_headers(headers)
        url = self.__urljoin(uri)
        res = requests.put(url, json=json_data, data=data, headers=_headers)
        return self.__handle_response(res)

    # @retry(wait=wait_exponential(max=3), stop=stop_after_attempt(3), reraise=True)
    def post(
        self,
        uri: str,
        json_data: Any = None,
        data: Any = None,
        headers: Optional[Dict] = None,
    ) -> Any:
        _headers = self._set_headers(headers)
        url = self.__urljoin(uri)
        res = requests.post(url, json=json_data, data=data, headers=_headers)
        return self.__handle_response(res)

    def _set_headers(self, headers: Dict[str, str]) -> Dict[str, str]:
        _headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.__api_key}",
        }
        if headers:
            _headers.update(headers)
        return _headers

    # @retry(wait=wait_exponential(max=3), stop=stop_after_attempt(3), reraise=True)
    def get(
        self, uri: str | None = None, *, params: Optional[Dict] = None, headers: Optional[Dict] = None
    ) -> Any:
        _headers = self._set_headers(headers)
        url = self.__urljoin(uri)
        translate_params = {}
        if params:
            for k, v in params.items():
                if isinstance(v, bool):
                    v = "true" if v else "false"
                translate_params[k] = v

        res = requests.get(url, params=translate_params, headers=_headers)
        return self.__handle_response(res)

    # @retry(wait=wait_exponential(max=3), stop=stop_after_attempt(3), reraise=True)
    def delete(
        self,
        uri: str,
        params: Optional[Dict] = None,
        headers: Optional[Dict] = None,
    ) -> Any:
        _headers = self._set_headers(headers)
        url = self.__urljoin(uri)
        translate_params = {}
        if params:
            for k, v in params.items():
                if isinstance(v, bool):
                    v = "true" if v else "false"
                translate_params[k] = v

        res = requests.delete(url, params=translate_params, headers=_headers)
        return self.__handle_response(res)

    def __urljoin(self, uri: str) -> str:
        if not uri:
            return self.__api_url
        return self.__api_url + uri

    @staticmethod
    def __handle_response(resp: Response) -> Any:
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as e:
            message = e.args[0]
            try:
                message += f", Response from server: {resp.text}"
            except ValueError:
                pass
            raise requests.exceptions.HTTPError(message, response=e.response)
        return resp.json() if resp.content else None

    def patch(
        self,
        uri: str,
        json_data: Any = None,
        data: Any = None,
        headers: Optional[Dict] = None,
    ) -> Any:
        _headers = self._set_headers(headers)
        url = self.__urljoin(uri)
        res = requests.patch(url, json=json_data, data=data, headers=_headers)
        return self.__handle_response(res)



"""
The module is used to interact with the Vendi Models API.
You can use this module to list, create, update, import models from huggingface and more.

Models can be used to create finetune that will be exposed as new endpoints to consume predictions from.
"""
from .models import Models


"""
Vendi is a Python client for the Vendi API. It provides a convenient way to access the Vendi API from a Python application.
This is the main entrypoint for the Vendi SDK.
"""
from vendi.core.config import vendi_config
from vendi.datasets import Datasets
from vendi.finetune import Finetune
from vendi.deployments.deployments import Deployments
from vendi.models import Models
from vendi.completions import Completions

VENDI_API_URL = vendi_config.VENDI_API_URL


class Vendi:
    def __init__(
        self,
        *,
        api_url: str | None = None,
        api_key: str | None = None,
    ):
        self._base_url = api_url or vendi_config.VENDI_API_URL

        if api_key:
            self.api_key = api_key
        else:
            self.api_key = vendi_config.VENDI_API_KEY

        self.models = Models(url=self._base_url, api_key=self.api_key)
        self.deployments = Deployments(url=self._base_url, api_key=self.api_key)
        self.datasets = Datasets(url=self._base_url, api_key=self.api_key)
        self.completions = Completions(url=self._base_url, api_key=self.api_key)
        self.finetune = Finetune(url=self._base_url, api_key=self.api_key)



"""
Deployments is an internal class that is used to interact with the Vendi Deployments API.
This one is used only for users with private deployments access.
Please contact us if you would like to use this feature.

Each deployment is associated with a single model and a single backend.
The model must be registered with vendi models and available to the user when running vendi.models.list().
"""
from .deployments import Deployments



from pydantic_settings import BaseSettings


class VendiConfig(BaseSettings):
    VENDI_API_KEY: str = ""
    """The API key to use when making requests to the Vendi API. Use the `VENDI_API_KEY` environment variable to set this value."""
    VENDI_API_URL: str = "https://api.vendi-ai.com"
    """The URL of the Vendi API. Use the `VENDI_API_URL` environment variable to set this value."""


vendi_config = VendiConfig()



import uuid
from datetime import datetime

from pydantic import BaseModel


class SchemaMixin(BaseModel):
    id: uuid.UUID
    """A unique identifier for the object."""
    created_at: datetime
    """The datetime timestamp the object was created."""
    updated_at: datetime
    """The datetime timestamp the object was last updated."""
    created_by: str
    """The user who created the object."""
    tenant: str
    """The tenant the object belongs to."""



import json
import uuid
from enum import Enum
from typing import Optional, Literal
from pydantic import BaseModel, model_validator, ConfigDict

from vendi.core.schema import SchemaMixin
from vendi.deployments.schema import DeploymentStatus


class ImageContentUrl(BaseModel):
    url: str


class ImageContent(BaseModel):
    type: Literal["image_url"]
    image_url: ImageContentUrl


class TextContent(BaseModel):
    type: Literal["text"]
    text: str


class LlmMessage(BaseModel):
    role: str
    """The role of the message. Can be `user` or `assistant`."""
    content: str | list[ImageContent | TextContent]
    """The content of the message."""


class CompletionUsage(BaseModel):
    completion_tokens: int
    """Number of tokens in the generated completion."""

    prompt_tokens: int
    """Number of tokens in the prompt."""

    total_tokens: int
    """Total number of tokens used in the request (prompt + completion)."""


class Choice(BaseModel):
    finish_reason: Literal["stop", "length"]
    """The reason the model stopped generating tokens.

    This will be `stop` if the model hit a natural stop point or a provided stop
    sequence, `length` if the maximum number of tokens specified in the request was
    reached.
    """

    index: int
    """The index of the choice in the list of choices."""

    message: LlmMessage
    """A chat completion message generated by the model."""


class ChatCompletion(BaseModel):
    id: str
    """A unique identifier for the chat completion."""

    choices: list[Choice]
    """A list of chat completion choices.

    Can be more than one if `n` is greater than 1.
    """

    created: int
    """The Unix timestamp (in seconds) of when the chat completion was created."""

    model: str
    """The model used for the chat completion."""

    object: Literal["chat.completion"] = "chat.completion"
    """The object type, which is always `chat.completion`."""

    usage: Optional[CompletionUsage] = None
    """Usage statistics for the completion request."""


class VendiCompletionResponse(ChatCompletion):
    id: str | None = None
    """A unique identifier for the completion."""
    elapsed_time: float | None = None
    """The total time taken to generate the completion."""
    provider: str | None = None
    """The name of the provider that served the completion."""
    parameters: any = None
    """The parameters used for the completion."""

    class Config:
        arbitrary_types_allowed = True


class CompletionParams(BaseModel):
    stop: str | list[str] = []
    """The stop sequence(s) to use for the completion."""
    temperature: float = 0.0
    """The sampling temperature to use for the completion."""
    max_tokens: int = 256
    """The maximum number of tokens to generate in the model response."""
    top_k: int = 40
    """The maximum number of tokens to consider for each step of the generation."""
    top_p: float = 0.95
    """The cumulative probability of the top tokens to consider for each step of the generation."""
    frequency_penalty: float = 1.0
    """The frequency penalty to use for the completion."""
    presence_penalty: float = 1.0
    """The presence penalty to use for the completion."""
    n: int = 1
    """The number of completions to generate."""
    json_schema: str | None = None
    """The json schema to enforce on the completion. Used for constrained generation"""
    regex: str | None = None
    """The regex to enforce on the completion. Used for constrained generation"""
    checkpoint: str | None = None
    """The checkpoint to use for the completion. Used for fine-tuned models by vendi"""

    @model_validator(mode="after")
    @classmethod
    def validate_stop(cls, data: "CompletionParams"):
        if isinstance(data.stop, str):
            data.stop = [data.stop]
        elif isinstance(data.stop, list):
            pass
        else:
            data.stop = []
        return data

    @model_validator(mode="after")
    @classmethod
    def validate_schema_and_regex(cls, data: "CompletionParams"):
        if data.json_schema and data.regex:
            raise ValueError(
                "Json schema and regex cannot be used together. Please use on of them"
            )
        if data.json_schema:
            try:
                json.loads(data.json_schema)
            except json.JSONDecodeError:
                raise ValueError(
                    "Passed schema is not valid json. Please check your syntax."
                )
        return data

    model_config = ConfigDict(protected_namespaces=(), extra='allow')


class ModelParameters(CompletionParams):
    model: str
    """The model to use for the completion - in the format <provider>/<model-name>"""


class CompletionRequest(ModelParameters):
    request_id: str | None = None
    """A unique identifier for the completion request. Can be used to track the request."""
    messages: list[LlmMessage]
    """The prompt messages to use for the completion."""


class BatchInferenceStatus(str, Enum):
    CREATING = "creating"
    """The batch inference is being created."""
    PENDING = "pending"
    """The batch job is created and waits for execution"""
    RUNNING = "running"
    """The batch job is running."""
    FAILED = "failed"
    """The batch job failed."""
    COMPLETED = "completed"
    """The batch job successfully completed."""


class BatchInference(SchemaMixin):
    model_parameters: list[ModelParameters]
    """The model parameters for the batch inference."""
    dataset_id: uuid.UUID
    """The dataset id for the batch inference."""
    output_dataset_id: uuid.UUID
    """The outputs of the batch job are saved in this dataset id. This should be used to retrieve the results."""
    status: BatchInferenceStatus
    """The status of the batch inference."""
    total_chunks: int | None = None
    """The batch job is split into chunks. This is the total number of chunks."""
    total_records: int | None = None
    """The total number of records in the batch inference."""
    completed_chunks: int | None = None
    """The number of completed chunks. This is updated as the batch job progresses"""
    job_name: str | None = None
    """The name of the batch job."""

    model_config = ConfigDict(
        protected_namespaces=(),
    )


class Endpoint(SchemaMixin):
    model_id: str
    """The model identifier served in this endpoint"""
    provider: str
    """The provider of the model (vendi/openai/google..)"""
    adapter_config: str | None = None
    """The lora adapter configuration for the model"""
    short_description: str | None = None
    """A description of the model"""
    full_description: str | None = None
    """A full description of the model"""
    docs: str | None = None
    """A doc link about the model"""
    status: DeploymentStatus | None = None
    """The status of the endpoint. If the endpoint is not ready, the status will be `not_ready`"""
    is_configured: bool | None = None
    """If the endpoint is configured by the user or not"""



from typing import Optional, List

from vendi.core.http_client import HttpClient
from vendi.finetune.schema import TrainData, TrainJob
from vendi.models.schema import ModelInfo, ModelProvider


class Finetune:
    def __init__(self, url: str, api_key: str):
        self.__client = HttpClient(
            url=url,
            api_key=api_key,
            api_prefix="/platform/v1"
        )

    def run(
        self,
        run_name: str,
        model_name: str,
        model_description: str,
        dataset_id: str,
        model_max_length: Optional[int] = 1024,
        split_eval: Optional[float] = 0.1,
        eval_steps: Optional[int] = 10,
        save_steps: Optional[int] = 50,
        per_device_train_batch_size: Optional[int] = 1,
        per_device_eval_batch_size: Optional[int] = 1,
        num_train_epochs: Optional[int] = 3,
        max_steps: Optional[int] = 50,
        save_total_limit: Optional[int] = 5,
        load_best_model_at_end: Optional[bool] = True,
        learning_rate: Optional[float] = 0.00002,
        gradient_accumulation_steps: Optional[int] = 1,
        warmup_steps: Optional[int] = 0,
        warmup_ratio: Optional[float] = 0.03,
        lora_r: Optional[int] = 32,
        lora_alpha: Optional[int] = 32,
        lora_dropout: Optional[float] = 0.05,
        lora_target_modules=None,
        q_lora: bool = True,
        compute_resource: Optional[str] = None,
    ) -> TrainJob:
        if lora_target_modules is None:
            lora_target_modules = [
                "q_proj",
                "o_proj",
                "up_proj",
                "v_proj",
                "gate_proj",
                "down_proj",
                "k_proj",
                "lm_head"
            ]

        res = self.__client.post(
            uri="/train/train-lora",
            json_data={
                "run_name": run_name,
                "model_name": model_name,
                "model_description": model_description,
                "dataset_id": dataset_id,
                "compute_resource": compute_resource,
                "training_params": {
                    "model_max_length": model_max_length,
                    "split_eval": split_eval,
                    "eval_steps": eval_steps,
                    "save_steps": save_steps,
                    "per_device_train_batch_size": per_device_train_batch_size,
                    "per_device_eval_batch_size": per_device_eval_batch_size,
                    "num_train_epochs": num_train_epochs,
                    "max_steps": max_steps,
                    "save_total_limit": save_total_limit,
                    "load_best_model_at_end": load_best_model_at_end,
                    "learning_rate": learning_rate,
                    "gradient_accumulation_steps": gradient_accumulation_steps,
                    "warmup_steps": warmup_steps,
                    "warmup_ratio": warmup_ratio,
                    "lora_r": lora_r,
                    "lora_alpha": lora_alpha,
                    "lora_dropout": lora_dropout,
                    "lora_target_modules": lora_target_modules,
                    "q_lora": q_lora
                }
            }
        )
        return TrainJob(**res)

    def logs(self, model_id: str) -> TrainData:
        """
        Get the logs for a finetune model by ID
        """
        res = self.__client.get(uri=f"/train/trainings/{model_id}/data")
        return TrainData(**res)

    def available_models(self, provider: ModelProvider = ModelProvider.VENDI) -> List[ModelInfo]:
        res = self.__client.get(uri=f"/providers/{provider}/models")
        return [ModelInfo(**model) for model in res[provider]]



import uuid
from enum import Enum
from typing import List

from pydantic import BaseModel, ConfigDict

from vendi.core.schema import SchemaMixin


class ModelType(str, Enum):
    PRE_TRAINED = "pre_trained"
    FULLY_TRAINED = "fully_trained"
    FINE_TUNED = "fine_tuned"
    EXTERNAL = "external"


class ModelProvider(str, Enum):
    VENDI = "vendi"
    """A model that is created by vendi for you"""
    HUGGINGFACE = "huggingface"
    """A model that is present on and was imported from huggingface"""
    TRAINING = "training"
    """A model that is created by finetuning a vendi model"""
    OPENAI = "openai"
    """A model that openai exposes through its api"""
    GOOGLE = "google"
    """A model that google exposes through its api"""


class ModelInfo(BaseModel):
    name: str
    """The name of the model"""
    source: ModelProvider
    """The provider of the model"""
    type_: ModelType
    """The type of the model"""
    model_config = ConfigDict(
        protected_namespaces=(),
    )


class Model(SchemaMixin):
    name: str
    source: str
    type_: str
    description: str
    is_public: bool
    base_model_id: uuid.UUID | str | None = None
    status: str | None


class HuggingFaceModel(BaseModel):
    hf_model_name: str
    hf_model_repo: str
    language: List[str]
    license: str
    tags: List[str]



import json
import logging
import uuid
from typing import List, Dict, Union, Optional

import pandas as pd

from vendi.core.http_client import HttpClient
from vendi.datasets.schema import Dataset, DatasetType

logger = logging.getLogger(__name__)


class Datasets:
    """
    Datasets is the client to interact with the datasets endpoint of the Vendi API.
    """

    def __init__(self, url: str, api_key: str):
        self.__client = HttpClient(
            url=url,
            api_key=api_key,
            api_prefix=f"/platform/v1/datasets"
        )

    def get(self, dataset_id: str) -> Dataset:
        """
        Get dataset metadata by ID
        :param dataset_id: The ID of the dataset to get
        """
        res = self.__client.get(uri=f"/{dataset_id}")
        dataset = Dataset(**res)

        def load_data() -> pd.DataFrame:
            """
            Download and load the dataset data into a pandas DataFrame
            """
            data = self.__client.get(uri=f"/{dataset_id}/data")
            dataset._data = pd.DataFrame(data['data'])
            return dataset.data

        dataset.load_data = load_data
        return dataset

    def list(self) -> list[Dataset]:
        """
        Get a list of the datasets available to the user
        :return: A list of datasets metadata
        """
        res = self.__client.get(uri=f"/")
        return [Dataset(**dataset) for dataset in res]

    def delete(self, dataset_id: str) -> None:
        """
        Delete a dataset by ID
        :param dataset_id: The ID of the dataset to delete
        """
        res = self.__client.delete(uri=f"/{dataset_id}")
        return res

    def upload(
        self,
        name: str,
        data: Optional[List[dict]] = None,
        data_path: Optional[str] = None,
        tags: Union[Dict, None] = None,
        path: Union[str, None] = None,
        dataset_type: Union[DatasetType, None] = None,
    ) -> Dataset:
        """
        Upload a dataset to the Vendi API
        :param name: The name of the dataset
        :param data: The data to upload. Should be a valid list of JSON objects
        :param data_path: The path to the data to upload. Should be a valid path to a JSON file
        :param tags: The tags to apply to the dataset. Should be a dictionary of key/value pairs. For example:
        {"version": "v3", "language": "en"}
        :param path: The path to the dataset. Should be a string, for example: "datasets/financial". Defaults to None
        :param dataset_type: The type of dataset to upload. Should be a DatasetType enum value.
        If the dataset is a conversation dataset, with 'messages' key use "conversation". If the dataset is a raw dataset columns, use "raw"
        :return: The ID of the uploaded dataset

        """
        if not data and not data_path:
            raise ValueError("You must provide either data or data_path when uploading data")

        if data_path:
            with open(data_path, "r") as f:
                try:
                    data = [json.loads(line) for line in f.read().splitlines()]
                except json.decoder.JSONDecodeError:
                    raise ValueError(f"Could not parse JSON from {data_path} file. The file must be a valid JSONL file")

        if tags is None:
            tags = {}
        str_data = ""
        for row in data:
            str_data += json.dumps(row) + "\n"

        if not dataset_type:
            if "messages" in data[0]:
                dataset_type = DatasetType.CONVERSATION
                logger.info("Detected conversation dataset, since there is a 'messages' key in the data")
            else:
                dataset_type = DatasetType.RAW
                logger.info("Detected raw dataset")

        res = self.__client.post(
            uri=f"/upload",
            json_data={
                "name": f"{name}.jsonl",
                "type": dataset_type,
                "tags": tags,
                "data": str_data,
                "path": path,
            }
        )
        return Dataset(**res)

    def generate_download_link(self, dataset_id: str) -> str:
        """
        Generate a download link for a dataset
        :param dataset_id: The ID of the dataset to download
        :return: The download link
        """
        res = self.__client.get(uri=f"/{dataset_id}/download-link")
        return res
